@@ -8,6 +8,8 @@
 
 import sys
 import os
+import time
+import pytest
 
 
 class SlaveInteractor:
@@ -33,11 +35,11 @@ class SlaveInteractor:
         slaveinfo = getinfodict()
         self.sendevent("slaveready", slaveinfo=slaveinfo)
 
-    def pytest_sessionfinish(self, __multicall__, exitstatus):
+    @pytest.hookimpl(hookwrapper=True)
+    def pytest_sessionfinish(self, exitstatus):
         self.config.slaveoutput['exitstatus'] = exitstatus
-        res = __multicall__.execute()
+        yield
         self.sendevent("slavefinished", slaveoutput=self.config.slaveoutput)
-        return res
 
     def pytest_collection(self, session):
         self.sendevent("collectionstart")
@@ -58,23 +60,29 @@ class SlaveInteractor:
             self.log("items to run:", torun)
             # only run if we have an item and a next item
             while len(torun) >= 2:
-                self.run_tests(torun)
+                self.run_one_test(torun)
             if name == "shutdown":
                 if torun:
-                    self.run_tests(torun)
+                    self.run_one_test(torun)
                 break
         return True
 
-    def run_tests(self, torun):
+    def run_one_test(self, torun):
         items = self.session.items
         self.item_index = torun.pop(0)
+        item = items[self.item_index]
         if torun:
             nextitem = items[torun[0]]
         else:
             nextitem = None
+
+        start = time.time()
         self.config.hook.pytest_runtest_protocol(
-            item=items[self.item_index],
+            item=item,
             nextitem=nextitem)
+        duration = time.time() - start
+        self.sendevent("runtest_protocol_complete", item_index=self.item_index,
+                       duration=duration)
 
     def pytest_collection_finish(self, session):
         self.sendevent(
@@ -96,25 +104,33 @@ class SlaveInteractor:
         data = serialize_report(report)
         self.sendevent("collectreport", data=data)
 
+    def pytest_logwarning(self, message, code, nodeid, fslocation):
+        self.sendevent("logwarning", message=message, code=code, nodeid=nodeid,
+                       fslocation=str(fslocation))
+
 
 def serialize_report(rep):
     def disassembled_report(rep):
-        reprtraceback = rep.longrepr.reprtraceback.__dict__
-        reprcrash = rep.longrepr.reprcrash.__dict__
+        reprtraceback = rep.longrepr.reprtraceback.__dict__.copy()
+        reprcrash = rep.longrepr.reprcrash.__dict__.copy()
 
         new_entries = []
         for entry in reprtraceback['reprentries']:
-            new_entry = entry.__dict__
-            for key, value in new_entry.items():
+            entry_data = {
+                'type': type(entry).__name__,
+                'data': entry.__dict__.copy(),
+            }
+            for key, value in entry_data['data'].items():
                 if hasattr(value, '__dict__'):
-                    new_entry[key] = value.__dict__
-            new_entries.append(new_entry)
+                    entry_data['data'][key] = value.__dict__.copy()
+            new_entries.append(entry_data)
 
         reprtraceback['reprentries'] = new_entries
 
         return {
             'reprcrash': reprcrash,
-            'reprtraceback': reprtraceback
+            'reprtraceback': reprtraceback,
+            'sections': rep.longrepr.sections
         }
 
     import py
@@ -162,10 +178,6 @@ def remote_initconfig(option_dict, args):
 
 if __name__ == '__channelexec__':
     channel = channel  # noqa
-    # python3.2 is not concurrent import safe, so let's play it safe
-    # https://bitbucket.org/hpk42/pytest/issue/347/pytest-xdist-and-python-32
-    if sys.version_info[:2] == (3, 2):
-        os.environ["PYTHONDONTWRITEBYTECODE"] = "1"
     slaveinput, args, option_dict = channel.receive()
     importpath = os.getcwd()
     sys.path.insert(0, importpath)  # XXX only for remote situations
